import logging
from datetime import datetime

from convopyro import listen_message
from pyrogram import filters
from pyrogram.types import ForceReply, Message, ReplyKeyboardRemove

from classes.pyroclient import PyroClient
from modules import custom_filters
from modules.utils import from_utc

logger = logging.getLogger(__name__)


@PyroClient.on_message(
    ~filters.scheduled & filters.private & filters.command(["set_offset"], prefixes=["/"]) & ~custom_filters.context  # type: ignore
)
async def command_set_offset(app: PyroClient, message: Message):
    user = await app.find_user(message.from_user)

    if user.location is None:
        await message.reply_text(
            app._("location_empty", "messages", locale=user.locale)
        )
        return

    await message.reply_text(
        app._("set_offset", "messages", locale=user.locale),
        reply_markup=ForceReply(
            placeholder=app._("set_offset", "force_replies", locale=user.locale)
        ),
    )

    while True:
        app.contexts.append(message.from_user.id)
        answer = await listen_message(app, message.chat.id, 300)
        app.contexts.remove(message.from_user.id)

        if answer is None or answer.text == "/cancel":
            await message.reply_text(
                app._("cancelled", "messages", locale=user.locale),
                reply_markup=ReplyKeyboardRemove(),
            )
            return

        try:
            num = int(answer.text)
            if num < 0 or num > 7:
                raise ValueError(
                    "Offset bust not be less than 0 and greater than 7 days."
                )
        except (ValueError, TypeError):
            await answer.reply_text(
                app._("set_offset_invalid", "messages", locale=user.locale).format(
                    cancel_notice=app._("cancel", "messages", locale=user.locale)
                )
            )
            continue

        break

    offset = int(answer.text)

    await user.update_offset(offset)

    logger.info("User %s has set offset to %s", user.id, offset)

    garbage_time = from_utc(
        datetime(1970, 1, 1, user.time_hour, user.time_minute),
        None if user.location is None else user.location.timezone.zone,
    ).strftime(app._("time", "formats"))

    await answer.reply_text(
        app._("set_offset_finished", "messages", locale=user.locale).format(
            offset=offset,
            time=garbage_time,
            toggle_notice=""
            if user.enabled
            else app._("toggle", "messages", locale=user.locale),
        ),
        reply_markup=ReplyKeyboardRemove(),
    )
